@@ -12,20 +12,32 @@ class PyrexError(Exception):
 class PyrexWarning(Exception):
     pass
 
-def context(position):
-    F = open(position[0]).readlines()
-    s = ''.join(F[position[1]-6:position[1]])
-    s += ' '*(position[2]-1) + '^'
-    s = '-'*60 + '\n...\n' + s + '\n' + '-'*60 + '\n'
-    return s
 
+def context(position):
+    source = position[0]
+    assert not (isinstance(source, unicode) or isinstance(source, str)), (
+        "Please replace filename strings with Scanning.FileSourceDescriptor instances %r" % source)
+    try:
+        F = list(source.get_lines())
+    except UnicodeDecodeError:
+        # file has an encoding problem
+        s = "[unprintable code]\n"
+    else:
+        s =''.join(F[max(0, position[1]-6):position[1]])
+        s = '...\n' + s + ' '*(position[2]-1) + '^\n'
+    s = '-'*60 + '\n' + s + '-'*60 + '\n'
+    return s
+    
 class CompileError(PyrexError):
     
     def __init__(self, position = None, message = ""):
         self.position = position
-        self.message = message
+        self.message_only = message
+        self.reported = False
+    # Deprecated and withdrawn in 2.6:
+    #   self.message = message
         if position:
-            pos_str = "%s:%d:%d: " % position
+            pos_str = "%s:%d:%d: " % (position[0].get_description(), position[1], position[2])
             cont = context(position)
         else:
             pos_str = ""
@@ -36,9 +48,10 @@ class CompileWarning(PyrexWarning):
     
     def __init__(self, position = None, message = ""):
         self.position = position
-        self.message = message
+    # Deprecated and withdrawn in 2.6:
+    #   self.message = message
         if position:
-            pos_str = "%s:%d:%d: " % position
+            pos_str = "%s:%d:%d: " % (position[0].get_description(), position[1], position[2])
         else:
             pos_str = ""
         Exception.__init__(self, pos_str + message)
@@ -76,21 +89,28 @@ def close_listing_file():
         listing_file.close()
         listing_file = None
 
-def error(position, message):
-    #print "Errors.error:", repr(position), repr(message) ###
+def report_error(err):
     global num_errors
-    err = CompileError(position, message)
+    # See Main.py for why dual reporting occurs. Quick fix for now.
+    if err.reported: return
+    err.reported = True
     line = "%s\n" % err
     if listing_file:
         listing_file.write(line)
     if echo_file:
         echo_file.write(line)
     num_errors = num_errors + 1
+
+def error(position, message):
+    #print "Errors.error:", repr(position), repr(message) ###
+    err = CompileError(position, message)
+    #if position is not None: raise Exception(err) # debug
+    report_error(err)
     return err
 
 LEVEL=1 # warn about all errors level 1 or higher
 
-def warning(position, message, level):
+def warning(position, message, level=0):
     if level < LEVEL:
         return
     warn = CompileWarning(position, message)
